import os
import sys
import datetime

def run(file_old:str, file_new:str, chr_old:str, chr_new:str, logfile='tmp.log'):
    os.system(f'axtChain -linearGap=medium -psl pyOverChainTmp.NewGenome.liftUp.{file_new}/{chr_new}.psl \
        -faT pyOverChainTmp.OldGenome.{file_old}/{chr_old}.fa \
            -faQ pyOverChainTmp.NewGenome.{file_new}/{chr_new}.fa \
                pyOverChainTmp.NewGenome.axtChain.{file_new}/{chr_new}.chain >>{logfile} 2>&1')

if __name__ == "__main__":
    run(file_old='ISR.fa', file_new='JGI.fa', chr_old='D03', chr_new='D03')


import os
import sys
import datetime

def run(file_new:str, chr_new:str, logfile='tmp.log'):

    os.system(f'faSplit -lift=pyOverChainTmp.NewGenome.lift.{file_new}/{chr_new}.lft \
        -oneFile size pyOverChainTmp.NewGenome.{file_new}/{chr_new}.fa 3000 \
            pyOverChainTmp.NewGenome.{file_new}/{chr_new}.split >>{logfile} 2>&1')

if __name__ == "__main__":
    run(file_new='JGI.fa',chr_new='D03')


import datetime

from pyoverchain.core import axtchain, blat, chainMergeSort, chainNet, faSplit, liftUp, netChainSubset

def run(args:dict):
    logfile = 'pyOverChainTmp.log/'+args['file_old']\
        +args['chr_old']+'-to-'+args['file_new']+args['chr_new']+'.log'

    flog = open(logfile, mode='w')

    print(f'[pyoverchain:{datetime.datetime.now()}] Begin faSplit...', file=flog, flush=True)
    faSplit.run(file_new=args['file_new'], chr_new=args['chr_new'], logfile=logfile)

    print(f'[pyoverchain:{datetime.datetime.now()}] Begin BLAT...', file=flog, flush=True)
    blat.run(file_old=args['file_old'], file_new=args['file_new'], \
        chr_old=args['chr_old'], chr_new=args['chr_new'], cpu=args['cpu'], binary=args['binary'], logfile=logfile)
    
    print(f'[pyoverchain:{datetime.datetime.now()}] Begin liftUp...', file=flog, flush=True)
    liftUp.run(file_new=args['file_new'], chr_new=args['chr_new'], logfile=logfile)

    print(f'[pyoverchain:{datetime.datetime.now()}] Begin axtchain...', file=flog, flush=True)
    axtchain.run(file_old=args['file_old'], file_new=args['file_new'], \
        chr_old=args['chr_old'], chr_new=args['chr_new'], logfile=logfile)
    
    print(f'[pyoverchain:{datetime.datetime.now()}] Begin chainMergeSort...', file=flog, flush=True)
    chainMergeSort.run(file_new=args['file_new'], chr_new=args['chr_new'], logfile=logfile)

    print(f'[pyoverchain:{datetime.datetime.now()}] Begin chainNet...', file=flog, flush=True)
    chainNet.run(file_old=args['file_old'], file_new=args['file_new'], \
        chr_new=args['chr_new'], logfile=logfile)

    print(f'[pyoverchain:{datetime.datetime.now()}] Begin netChainSubset...', file=flog, flush=True)
    netChainSubset.run(file_new=args['file_new'], chr_new=args['chr_new'], logfile=logfile)
    

if __name__ == "__main__":
    run({
        'file_old': 'ISR.fa',
        'file_new': 'JGI.fa', 
        'chr_old': 'D03', 
        'chr_new': 'D03', 
        'cpu': 50, 
        'binary': 'pblat'
    })


import os
import sys
import datetime

def run(file_old:str, file_new:str, chr_old:str, chr_new:str,  cpu=1, binary='blat', logfile='tmp.log'):
    if binary=='pblat':
        cmd = f'pblat pyOverChainTmp.OldGenome.{file_old}/{chr_old}.fa \
            pyOverChainTmp.NewGenome.{file_new}/{chr_new}.split.fa \
                pyOverChainTmp.NewGenome.blat.{file_new}/{chr_new}.psl -threads={cpu} \
                    -tileSize=12 -minScore=100 -minIdentity=98 -fastMap'
    else:
        cmd = f'blat pyOverChainTmp.OldGenome.{file_old}/{chr_old}.fa \
            pyOverChainTmp.NewGenome.{file_new}/{chr_new}.split.fa \
                pyOverChainTmp.NewGenome.blat.{file_new}/{chr_new}.psl \
                    -tileSize=12 -minScore=100 -minIdentity=98 -fastMap'
    os.system(cmd+f' >>{logfile} 2>&1')

if __name__ == "__main__":
    run(file_old='ISR.fa', file_new='JGI.fa', chr_old='D03', chr_new='D03', cpu=50, binary='pblat')


import os
import sys
import datetime

def run(file_new:str, chr_new:str, file_old:str, logfile='tmp.log'):
    os.system(f'cut -f 1,2 {file_old}.fai >pyOverChainTmp.OldGenome.{file_old}/chr.sizes')
    os.system(f'cut -f 1,2 {file_new}.fai >pyOverChainTmp.NewGenome.{file_new}/chr.sizes')
    os.system(f'chainNet pyOverChainTmp.NewGenome.chain.{file_new}/{chr_new}.chain \
        pyOverChainTmp.OldGenome.{file_old}/chr.sizes pyOverChainTmp.NewGenome.{file_new}/chr.sizes \
            pyOverChainTmp.NewGenome.net.{file_new}/{chr_new}.net /dev/null >>{logfile} 2>&1')

if __name__ == "__main__":
    run(file_new='JGI.fa',chr_new='D03', file_old='ISR.fa')


import multiprocessing as mp
import time
import datetime
import sys
import os

from glob import glob

import progressbar

from pyoverchain.core import pipeline_single, chr_separation

def run(file_old:str, file_new:str, chr_map:dict, num_parallel_chr:int, \
    num_thread_blat:int, binary:str, noprogress=False):
    # separate chromosomes
    chr_separation.run(file_old=file_old, file_new=file_new, chr_map=chr_map)

    # distribute tasks
    print(f'[pyoverchain:{datetime.datetime.now()}] Begin building chains by {num_parallel_chr} parallel tasks...', \
        file=sys.stderr)
    pool = mp.Pool(processes=num_parallel_chr)
    multi_res = []
    for (chr_old, chr_new) in chr_map.items():
        args = {
            'file_old': file_old,
            'file_new': file_new, 
            'chr_old': chr_old, 
            'chr_new': chr_new, 
            'cpu': num_thread_blat, 
            'binary': binary
        }
        multi_res.append(pool.apply_async(pipeline_single.run, (args,)))
    
    # monitor
    if noprogress is False:
        widgets = ['Running: ', progressbar.Counter(),\
                    ' chromosomes Finished', ' (', progressbar.Percentage(), ')', \
                        progressbar.Bar(), progressbar.ETA()]
        bar = progressbar.ProgressBar(widgets=widgets, max_value=len(chr_map)).start()
    while True:
        complete_count = sum([1 for x in multi_res if x.ready()])
        if complete_count == len(chr_map):
            if noprogress is False:
                bar.finish()
            break
        if noprogress is False:
            bar.update(complete_count)
        time.sleep(1)
    
    # Combine
    print(f'[pyoverchain:{datetime.datetime.now()}] Combining Results...', file=sys.stderr)
    chain_files = ' '.join(glob(f'pyOverChainTmp.NewGenome.over.{file_new}/*.chain'))
    os.system(f'cat {chain_files} >{file_old}.to.{file_new}.over.chain')



if __name__ == "__main__":
    run(file_old='ISR.fa', file_new='JGI.fa', chr_map={'D03':'D03', 'D02':'D02'}, \
        num_parallel_chr=2, num_thread_blat=50, binary='pblat')


import os
import sys
import datetime

def run(file_old:str, file_new:str, chr_map:dict):
    for (chr_old, chr_new) in chr_map.items():
        print(f'[pyoverchain:{datetime.datetime.now()}] Begin separating {chr_old} from {file_old}...', file=sys.stderr)
        os.system(f'samtools faidx {file_old} {chr_old} >pyOverChainTmp.OldGenome.{file_old}/{chr_old}.fa')

        print(f'[pyoverchain:{datetime.datetime.now()}] Begin separating {chr_new} from {file_new}...', file=sys.stderr)
        os.system(f'samtools faidx {file_new} {chr_new} >pyOverChainTmp.NewGenome.{file_new}/{chr_new}.fa')

if __name__ == "__main__":
    run(file_old='ISR.fa', file_new='JGI.fa', chr_map={'D03':'D03'})



import os
import sys
import datetime

def run(file_new:str, chr_new:str, logfile='tmp.log'):
    os.system(f'liftUp -pslQ pyOverChainTmp.NewGenome.liftUp.{file_new}/{chr_new}.psl \
        pyOverChainTmp.NewGenome.lift.{file_new}/{chr_new}.lft warn \
            pyOverChainTmp.NewGenome.blat.{file_new}/{chr_new}.psl >>{logfile} 2>&1')

if __name__ == "__main__":
    run(file_new='JGI.fa',chr_new='D03')


from setuptools import setup, find_packages

from pyoverchain.core import version

setup(name='pyoverchain',
      version=version.get(),
      description="a python pipeline to generate chain files between different genome assemblies for LiftOver",
      long_description=__doc__,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: MIT License',
          'Operating System :: Unix',
          'Programming Language :: Python :: 3.6',
          'Topic :: Scientific/Engineering :: Bio-Informatics'
      ],
      keywords='genome bioinformatics assembly BLAT LiftOver',
      author='Tao Zhu',
      author_email='taozhu@mail.bnu.edu.cn',
      url='https://github.com/billzt/pyOverChain',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      python_requires='>=3.6',
      install_requires=[
          'progressbar2'
      ],
      entry_points={
          'console_scripts': [
              'pyoverchain = pyoverchain.cmd.run:main',
          ]
      }
)


#!/usr/bin/env python3

'''pyOverChain: a python pipeline to generate chain files between different genome assemblies for LiftOver.
Github: https://github.com/billzt/pyOverChain
'''

__author__ = 'Tao Zhu'
__copyright__ = 'Copyright 2020'
__license__ = 'GPL'
__version__ = '0.1'
__email__ = 'taozhu@mail.bnu.edu.cn'
__status__ = 'Development'

import argparse
import re
import sys
import shutil
import os

from distutils.version import LooseVersion
from operator import itemgetter

from pyoverchain.core import version
from pyoverchain.core import pipeline_multiple

def make_args():
    parser = argparse.ArgumentParser(description='pyOverChain: a python pipeline to generate \
        chain files between different genome assemblies for LiftOver.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s '+version.get())
    parser.add_argument('old_genome', help='Old genome file in FASTA format')
    parser.add_argument('new_genome', help='New genome file in FASTA format')
    parser.add_argument('chr_map', help='A chromosome-mapping-file in TSV format', type=argparse.FileType('r'))
    parser.add_argument('-n', '--num-chromosome-tasks', help='number of parallel tasks to run different \
        chromosomes', type=int, default=2)
    parser.add_argument('-p', '--num-threads-pblat', help='number of threads for pblat. \
        Only useful if pblat were available', type=int, default=2)
    parser.add_argument('--disable-progress', help='Disable progress displaying. Useful if you want to \
        run it in background', action='store_true')
    args = parser.parse_args()
    return args

def check_environments():
    for i in ['samtools', 'faSplit', 'liftUp', 'axtChain', 'chainMergeSort', 'chainNet', 'netChainSubset']:
        if shutil.which(i) is None:
            raise Exception(f'No {i} detected in your system')
    blat_binary = ''
    if shutil.which('pblat') is not None:
        blat_binary = 'pblat'
    elif shutil.which('blat') is not None:
        blat_binary = 'blat'
    if blat_binary=='':
        raise Exception(f'Neither blat nor pblat detected in your system')
    return blat_binary


def main():
    args = make_args()
    blat_binary = check_environments()

    # chr_map
    chr_map = {}
    for line in args.chr_map:
        if line.startswith('#'):
            continue
        (chr_old, chr_new) = line.strip().split()
        chr_map[chr_old] = chr_new

    # make directories
    os.system("rm -rf pyOverChainTmp.*")
    os.system(f"mkdir pyOverChainTmp.log pyOverChainTmp.NewGenome.axtChain.{args.new_genome} \
        pyOverChainTmp.NewGenome.blat.{args.new_genome} pyOverChainTmp.NewGenome.chain.{args.new_genome} \
            pyOverChainTmp.NewGenome.{args.new_genome} pyOverChainTmp.NewGenome.lift.{args.new_genome} \
                pyOverChainTmp.NewGenome.liftUp.{args.new_genome} pyOverChainTmp.NewGenome.net.{args.new_genome} \
                    pyOverChainTmp.NewGenome.over.{args.new_genome} pyOverChainTmp.OldGenome.{args.old_genome}")

    # run
    pipeline_multiple.run(file_old=args.old_genome, file_new=args.new_genome, chr_map=chr_map, \
        num_parallel_chr=args.num_chromosome_tasks, num_thread_blat=args.num_threads_pblat, \
            binary=blat_binary, noprogress=args.disable_progress)
    
    # remove directories
    os.system("rm -rf pyOverChainTmp.*")

if __name__ == "__main__":
    main()


import os
import sys
import datetime

def run(file_new:str, chr_new:str, logfile='tmp.log'):
    os.system(f'chainMergeSort pyOverChainTmp.NewGenome.axtChain.{file_new}/{chr_new}.chain \
         | chainSplit pyOverChainTmp.NewGenome.chain.{file_new} stdin >>{logfile} 2>&1')

if __name__ == "__main__":
    run(file_new='JGI.fa',chr_new='D03')